#------------------------------------------------------------------
# $Copyright (c) 2017 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other
# names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $
#
# $Header: /project/isv-agent/VProAgents/Informatica/Informatica.pm,v 1.18 2013/03/21 10:00:49 sgadgil Exp $
#
# $Author: sgadgil $
#
# $Date: 2013/03/21 10:00:49 $
# $Revision: 1.18 $
#
# Provides:                            Informatica
# Author:                              sgadgil
# crAgent Build:                       \$Id: crAgent,v 1.99 2012
# crAgent Run Date:                    23 Jul, 2012
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------

#------------
# Be strict..
#------------
use strict;

#----------------------
# Declare the package..
#----------------------
package Informatica;

use ag_i18n_inc;
use POSIX;

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # Support generalized UNIX commands
use Proc;           # For calls to the process routines
use Comms;          # For calls to socket(3C) routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses
use SystemD;        # Support generalized SystemD Functions

#--------------------------------
# Include standard PERL modules..
#--------------------------------
use Fcntl;
use Socket;
use File::Path qw ( mkpath );
use File::Basename;
use File::stat;
use HTTP::Request;
use LWP::UserAgent;

my ( $CLUS_HOME, $sPrefix ) = ();

#------------------------
# Setup VCS/VCSone home..
#------------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.x/VCSOne..
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else { # VCS 4.x..
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /.*(vad|vcs(|one))$/;

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
my $sCmdHares  = $HARES  ||= "$CLUS_HOME/bin/hares";
my $sCmdHatype = $HATYPE ||= "$CLUS_HOME/bin/hatype";
my @liRunningProcs = (); # Global..

#-----------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested..
#-----------------------------------------------------------------------
my $sVer = "\$Id: Informatica.pm,v 1.18 2013/03/21 10:00:49 sgadgil Exp $::ETVHeader";
Version::TellVersion ( $sVer );

#--------------------------------
# Define Informatica subroutines..
#--------------------------------

=pod

 -------------------------------------------------------------------------
 Function :  bool ArgsValid ( IN String EntryPointName,
                              IN String User,
                              IN String Component,
                              IN String EnvFile,
                              IN String MonitorProgram,
                              IN String Hostname,
                              IN Integer Port,
                              IN Integer SecondLevelMonitor,
                              IN String DXHome,
                              IN String ActiveMQHome,
                              IN String MFTHome,
                              IN String InfaHome );

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName               Entry pt name
 $localUser                     System user credentials used to start the program
 $localComponent                Component for which the resource should be configured
 $localEnvFile                  Absolute path to environment file to source
 $localMonitorProgram           Full path to external monitor program
 $localHostname                 Hostname on which the resource is configured
 $localPort                     Port to be used by the resource for startup, monitor, etc.
 $localSecondLevelMonitor       Cycle count to run in-depth monitoring
 $localDXHome                   Absolute path to the B2B Data Exchange installation directory
 $localActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $localMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $localInfaHome                 Absolute path to the PowerCenter installation directory.
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ArgsValid {
  #----------------------------------------
  # Set up local vars from passed in ones..
  #----------------------------------------
  my ( $sEntryPointName,
       $localUser,
       $localComponent,
       $localEnvFile,
       $localMonitorProgram,
       $localHostname,
       $localPort,
       $localSecondLevelMonitor,
       $localDXHome,
       $localActiveMQHome,
       $localMFTHome,
       $localInfaHome ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  User                     [%s]
  Component                [%s]
  EnvFile                  [%s]
  MonitorProgram           [%s]
  Hostname                 [%s]
  Port                     [%s]
  SecondLevelMonitor       [%s]
  DXHome                   [%s]
  ActiveMQHome             [%s]
  MFTHome                  [%s]
  InfaHome                 [%s]",

  $sMyName,    $sEntryPointName,    $localUser,    $localComponent,
  $localEnvFile, $localMonitorProgram, $localHostname,  $localPort,
  $localSecondLevelMonitor,    $localDXHome,    $localActiveMQHome,
  $localMFTHome, $localInfaHome );

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName User  Component  EnvFile  MonitorProgram  Hostname
       Port SecondLevelMonitor DXHome ActiveMQHome MFTHome InfaHome );

  #---------------------------------
  # list of supported components ...
  #---------------------------------
  my @lsSupportedComponents = qw ( activemq dxconsole dxserver mft powercenter );

  @hsArgList { @lsArgs } = @_; # Assign..

  #----------------------------------------
  # Reqd. attributes based on entry point..
  #----------------------------------------
  my (@lsOnlineArgs, @lsOfflineArgs, @lsMonitorArgs, @lsImfRegisterArgs, @lsCleanArgs) = ();
  
  @lsOnlineArgs  = qw ( User Component Hostname Port );
  @lsOfflineArgs = qw ( User Component Hostname Port );
  @lsMonitorArgs = qw ( User Component Hostname Port );
  @lsImfRegisterArgs = qw ( User Component Hostname Port );

  if ( $localComponent eq "activemq" ) {
    push ( @lsMonitorArgs, qw ( ActiveMQHome DXHome ) );
    push ( @lsImfRegisterArgs, qw ( ActiveMQHome DXHome ) );

  } elsif ( $localComponent eq "dxconsole" ) {
    push ( @lsMonitorArgs, qw ( DXHome ) );
    push ( @lsImfRegisterArgs, qw ( DXHome ) );

  } elsif ( $localComponent eq "dxserver" ) {
    push ( @lsMonitorArgs, qw ( DXHome ) );
    push ( @lsImfRegisterArgs, qw ( DXHome ) );

  } elsif ( $localComponent eq "mft" ) {
    push ( @lsMonitorArgs, qw ( DXHome MFTHome ) );
    push ( @lsImfRegisterArgs, qw ( DXHome MFTHome ) );

  } elsif ( $localComponent eq "powercenter" ) {
    push ( @lsMonitorArgs, qw ( InfaHome ) );
    push ( @lsImfRegisterArgs, qw ( InfaHome ) );
  }

  @lsCleanArgs   = qw ( User Component Hostname Port );

  my %hlsNeeded = ( 'online'       => \@lsOnlineArgs,
                    'offline'      => \@lsOfflineArgs,
                    'monitor'      => \@lsMonitorArgs,
                    'clean'        => \@lsCleanArgs,
                    'imf_register' => \@lsImfRegisterArgs );

  my @lsReqdArgs = @{ $hlsNeeded { $sEntryPointName } };

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20002,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; # Failure..
  }

  #--------------------------------
  # Check for invalid component ...
  #--------------------------------
  if ( ! grep ( /$localComponent/, @lsSupportedComponents ) ) {
    &logError ( 20171,
      "<%s> Component [%s] is an invalid or unsupported component",
      $sMyName, $localComponent);

    $iReturnCode = 0; # Failure..
  }
  
  $iReturnCode || return $iReturnCode; # Return, if failed..

  #-----------------------------
  # Remove SLM file, if exists..
  #-----------------------------
  if ( $sEntryPointName ne 'monitor' ) {
    VCSagentFW::RunSecondLevelMonitor ( 1 ); # Delete..
  }

  #-------------------------------
  # Individual validation starts..
  #-------------------------------

  #----------------------------------
  # Validation for all entry points..
  #----------------------------------

  #------------------------
  # Validate parameter User..
  #------------------------
  if ( ! defined getpwnam $localUser ) {
    &logError ( 20003,
      "<%s> User [%s] is not a valid UNIX login",
    $sMyName, $localUser );
    $iReturnCode = 0; # Failure..
  }

  #----------------------------
  # Validate parameter Hostname..
  #----------------------------
  if ( ! inet_aton $localHostname ) {
    &logError ( 20004,
      "<%s> Hostname [%s] is an invalid Hostname/IPv4 address",
    $sMyName, $localHostname );
    $iReturnCode = 0; # Failure..
  }

  #-------------------------------------------------------
  # Validation for 'online' and 'offline' entry points ...
  #-------------------------------------------------------

  if ( $sEntryPointName eq "online" ||
       $sEntryPointName eq "offline" ) {
    #--------------------------------
    # Validate parameter ActiveMQHome..
    #--------------------------------
    if ($localComponent eq "activemq") {
      if ( ! -d $localActiveMQHome ) {
        &logError ( 20005,
        "<%s> ActiveMQHome [%s] directory does not exist",
        $sMyName, $localActiveMQHome );
        $iReturnCode = 0; # Failure..
      }
    }

    #--------------------------
    # Validate parameter Port..
    #--------------------------
    if ( $localComponent eq "dxserver" ) {
      if ( ! $localPort ) {
        &logError ( 20166, "<%s> Port [%s] has to be greater than 0",
        $sMyName, $localPort );
        $iReturnCode = 0; # Failure..
      }
    }

    #----------------------------
    # Validate parameter DXHome..
    #----------------------------
    if ($localComponent eq "activemq" ||
          $localComponent eq "dxconsole" || 
          $localComponent eq "dxserver" ) {
          if ( ! -d $localDXHome ) {
              &logError ( 20006,
                  "<%s> DXHome [%s] directory does not exist",
                  $sMyName, $localDXHome );
              $iReturnCode = 0; # Failure..
          }
      }

      #----------------------------
      # Validate parameter InfaHome..
      #----------------------------
      if ($localComponent eq "powercenter") {
          if ( ! -d $localInfaHome ) {
              &logError ( 20007,
                  "<%s> InfaHome [%s] directory does not exist",
                  $sMyName, $localInfaHome );
              $iReturnCode = 0; # Failure..
          }
      }

      #---------------------------
      # Validate parameter MFTHome..
      #---------------------------
      if ($localComponent eq "mft") {
          if ( ! -d $localMFTHome ) {
              &logError ( 20008,
                  "<%s> MFTHome [%s] directory does not exist",
                  $sMyName, $localMFTHome );
              $iReturnCode = 0; # Failure..
          }
      }
  }

  #---------------------------------------
  # Validation for online entry point(s)..
  #---------------------------------------
  if ( $sEntryPointName eq "online" ) {
    #------------------------------------
    # Validate parameter MonitorProgram..
    #------------------------------------
    if ( $localMonitorProgram =~ /(\S+)/ && # Defined..
         ! -x $1 ) {
      &logError ( 20009,
      "<%s> MonitorProgram [%s] is not executable",
      $sMyName, $1 );
      $iReturnCode = 0; # Failure..
    }
  }

  $iReturnCode ||
  &logWarn ( 20010,
  "<%s> Some of the attributes could not be validated. Returning [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode ;
}



=pod

 -------------------------------------------------------------------------
 Function : bool Start ( IN String ActiveMQHome,
                         IN String Component,
                         IN String DXHome,
                         IN String EnvFile,
                         IN String Hostname,
                         IN String InfaHome,
                         IN String MFTHome,
                         IN Integer Port,
                         IN Integer SecondLevelMonitor,
                         IN String User );

 Description : Start logic needed for online entry pt

 Parameters :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sComponent                Component for which the resource should be configured
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sEnvFile                  Absolute path to environment file to source
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $iPort                     Port to be used by the resource for startup, monitor, etc.
 $iSecondLevelMonitor       Cycle count to run in-depth monitoring
 $sUser                     System user credentials used to start the program
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Start {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sEnvFile,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $iPort,
       $iSecondLevelMonitor,
       $sUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20011, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  EnvFile                  [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  Port                     [%s]
  SecondLevelMonitor       [%s]
  User                     [%s]",

  $sMyName,  $sActiveMQHome,  $sComponent,   $sDXHome,   $sEnvFile,
  $sHostname, $sInfaHome, $sMFTHome, $iPort,  $iSecondLevelMonitor,
  $sUser );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;
  
  #-----------------------------------------
  # Set the file name to save command output
  #-----------------------------------------
  my $sResourceType   = $ENV{'VCS_LOG_AGENT_NAME'};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};

  my $sOutputFilename    = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.out";
  
  if (-f "$sOutputFilename") {
    if (0 == unlink ("$sOutputFilename") ) {
      &logError ( 20167,
        "<%s> Unable to delete output file [%s]",
        $sMyName, $sOutputFilename );
      return 0;
    }
  }

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sCommand, $sCommandArgs, $iTillWhen, $iExitCode, $iTimeout,
       @lsCmdout, @liPids ) = ();

  #-----------------------------------------
  # Whack any preexisting Informatica processes..
  #-----------------------------------------
  &logTrace ( 20012,
  "<%s> Killing Informatica processes for this instance", $sMyName );

  if ( Informatica::KillInformaticaProcesses ( $sActiveMQHome,
                                               $sComponent,
                                               $sDXHome,
                                               $sHostname,
                                               $sInfaHome,
                                               $sMFTHome,
                                               $sUser ) ) {
    &logTrace ( 20013,
    "<%s> Processes for this instance do not exist before start-up",
    $sMyName );
  }
  else {
    &logError ( 20014,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );
    $iReturnCode = 0; # Failure..
  }
  
  #--------------------
  # Return, if failed..
  #--------------------
  $iReturnCode || return $iReturnCode;

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 );
  
  #------------------------------------------
  # Define start command and args variables..
  #------------------------------------------

  if ($sComponent eq "activemq" ) {
    $sCommand = "$sDXHome/bin/activemq.sh";
    $sCommandArgs = "start";
  } elsif ($sComponent eq "dxconsole") {
    $sCommand = "$sDXHome/bin/dxconsole.sh";
    $sCommandArgs = "start";
  } elsif ($sComponent eq "dxserver") {
    $sCommand = "$sDXHome/bin/dxserver.sh";
    $sCommandArgs = "start $iPort";
  } elsif ($sComponent eq "mft") {
    $sCommand = "$sDXHome/bin/mft.sh";
    $sCommandArgs = "start";
  } elsif ($sComponent eq "powercenter") {
    $sCommand = "$sInfaHome/server/tomcat/bin/infaservice.sh";
    $sCommandArgs = "startup";
  }

  ( @lsCmdout, $iExitCode, $iTimeout ) = ();

  &logTrace ( 20015,
  "<%s> Starting Informatica instance with Cmd [%s] Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  my $bSourceUserEnv = 0; # Do not source..
  my $bTimedOut = 0;      # Timeout variable..

  #------------------
  # Request startup..
  #------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20016,
    "<%s> Insufficient time to continue boot up. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = 0; # Failure..
  }
  elsif( $::gbUseSystemD && SystemD::DetectSystemd() ) {  
    my $sCommandName = "simple";
    if ( $iReturnCode = SystemD::RunSystemdWithEnv($sUser, $sCommand, $sCommandArgs, $sEnvFile, $iTimeout, $sCommandName, undef ) ) { 
        &logTrace ( 20022,
        "<%s> Cmd [%s] with args [%s] executed successfully",
        $sMyName, $sCommand, $sCommandArgs );
      }    
  }
  elsif ( $iReturnCode =
      Sys::RunWithEnvCmdInBG ( $sEnvFile,
                              \$bSourceUserEnv,
                              $sCommand,
                              $sCommandArgs,
                              \$iExitCode,
                              $sUser,
                              $sOutputFilename,
                              \$bTimedOut,
                              $iTimeout,
                              dirname($sCommand) ) ) {
    &logTrace ( 20017,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );
  }

  #-------------------
  # Return if failed..
  #-------------------
  $iReturnCode || return $iReturnCode;

  #--------------------
  # Delay entry point..
  #--------------------
  my $iSleepTime = 10; # Secs..
  
  while ( ( $iTimeout = $iTillWhen - time ) > 10 ) {
    @liPids = (); # Initialise..

    sleep $iSleepTime; # Sec..

    &logTrace ( 20018,
    "<%s> Confirming start-up with [%s] sec in hand",
    $sMyName, $iTimeout - $iSleepTime );

    #---------
    # Do FLM..
    #---------
    if ( VCSDef_MonitorExit::ONLINE ==
         Informatica::GetProcState ( \@liPids,
                                     $sActiveMQHome,
                                     $sComponent,
                                     $sDXHome,
                                     $sHostname,
                                     $sInfaHome,
                                     $sMFTHome,
                                     $sUser,
                                     1 ) && (                # Running FFLM..
         $iSecondLevelMonitor  ?                             # If enabled..
         VCSDef_MonitorExit::ONLINE ==                       # Online after..
         Informatica::SecondLevelMonitor ( $sActiveMQHome,   # Running SLM..
                                           $sComponent,
                                           $sDXHome,
                                           $sEnvFile,
                                           $sHostname,
                                           $sInfaHome,
                                           $sMFTHome,
                                           $iPort,
                                           $sUser,
                                           $iTillWhen ) : 1 ) ) { 
      #---------------------
      # Start-up confirmed..
      #---------------------
      &logInfo ( 20019,
      "<%s> Confirmed start-up of this instance of Informatica (%s)",
      $sMyName, $sComponent );

      $iReturnCode = 1; # Success..
      last;
    }
    else {
      &logTrace ( 20020,
      "<%s> Could not confirm start-up. Sleeping for [%s] sec",
      $sMyName, $iSleepTime );

      $iReturnCode = 0; # Failure..
    }
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Stop ( IN String ActiveMQHome,
                        IN String Component,
                        IN String DXHome,
                        IN String EnvFile,
                        IN String Hostname,
                        IN String InfaHome,
                        IN String MFTHome,
                        IN Integer Port,
                        IN String User );

 Description : Stop logic needed for offline entry pt

 Parameters :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sComponent                Component for which the resource should be configured
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sEnvFile                  Absolute path to environment file to source
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $iPort                     Port to be used by the resource for startup, monitor, etc.
 $sUser                     System user credentials used to start the program
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Stop {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sEnvFile,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $iPort,
       $sUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20021, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  EnvFile                  [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  Port                     [%s]
  User                     [%s]",

  $sMyName,  $sActiveMQHome,  $sComponent,   $sDXHome,   $sEnvFile,
  $sHostname, $sInfaHome, $sMFTHome, $iPort, $sUser );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..
  
  #-----------------------------------------
  # Set the file name to save command output
  #-----------------------------------------
  my $sResourceType   = $ENV{'VCS_LOG_AGENT_NAME'};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};

  my $sOutputFilename    = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.out";

  if (-f "$sOutputFilename") {
    if (0 == unlink ("$sOutputFilename") ) {
      &logError ( 20168,
        "<%s> Unable to delete output file [%s]",
        $sMyName, $sOutputFilename );
      return 0;
    }
  }

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 ); # With margin..

  #------------------------------------------
  # Declare stop command and args variables..
  #------------------------------------------
  my ( $sCommand, $sCommandArgs, $iTimeout, $iExitCode, $iSleepTime,
       @lsCmdout, @liPids ) = ();
  
  #------------------------------------
  # Define stop command and arguments..
  #------------------------------------
  if ($sComponent eq "activemq") {
    $sCommand = "$sDXHome/bin/activemq.sh";
    $sCommandArgs = "stop";
  } elsif ($sComponent eq "dxconsole") {
    $sCommand = "$sDXHome/bin/dxconsole.sh";
    $sCommandArgs = "stop";
  } elsif ($sComponent eq "dxserver") {
    $sCommand = "$sDXHome/bin/dxserver.sh";
    $sCommandArgs = "stop $sHostname $iPort";
  } elsif ($sComponent eq "mft") {
    $sCommand = "$sDXHome/bin/mft.sh";
    $sCommandArgs = "stop";
  } elsif ($sComponent eq "powercenter") {
    $sCommand = "$sInfaHome/server/tomcat/bin/infaservice.sh";
    $sCommandArgs = "shutdown";
  }

  ( $iExitCode, $iTimeout, @lsCmdout ) = (); # Truncate.. 

  &logTrace ( 20172,
  "<%s> Stopping Informatica with Command [%s] Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  #------------------------
  # Request shutdown..
  #------------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20023,
    "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = 0; # Failure..
  }
  elsif ( $iReturnCode =
          Informatica::ExecLocalCmd ( $sUser, 
                                      $sEnvFile, 
                                      $sCommand, 
                                      $sCommandArgs, 
                                      \$iExitCode, 
                                      $iTimeout, 
                                      \@lsCmdout, 
                                      $sOutputFilename ) ) {
    &logTrace ( 20024,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );
  }


  #--------------------
  # Delay entry point..
  #--------------------
  if ( $iReturnCode ) { # Success..
    #-----------------
    # Set sleep time..
    #-----------------
    $iSleepTime = 10; # Sec..
  
    &logInfo ( 20025,
    "<%s> Delaying exit to confirm shutdown..", $sMyName );

    #--------------------
    # Delay entry point..
    #--------------------
    while ( ( $iTimeout = $iTillWhen - time ) > 20 ) {
      #-----------------------
      # Wait before retrying..
      #-----------------------
      sleep $iSleepTime; # Secs..
  
      Informatica::GetProcState ( \@liPids,
                                  $sActiveMQHome,
                                  $sComponent,
                                  $sDXHome,
                                  $sHostname,
                                  $sInfaHome,
                                  $sMFTHome,
                                  $sUser,
                                  0 ); # Request regular..
  
      #------------------------------
      # Check for pending processes..
      #------------------------------
      if ( scalar @liPids ) {
        &logTrace ( 20026,
        "<%s> Could not confirm shutdown. Retrying in [%s] sec.
        Delaying kill with [%s] sec in hand",
        $sMyName, $iSleepTime, $iTimeout - $iSleepTime );
      }
      else {
        &logInfo ( 20027,
        "<%s> Confirmed shutdown of Informatica instance",
        $sMyName );
        last; # Get out..
      }
    }
  }

  #-----------------------------------------
  # Whack any persisting Informatica processes..
  #-----------------------------------------
  &logTrace ( 20028,
  "<%s> Killing Informatica processes for this instance", $sMyName );

  if ( Informatica::KillInformaticaProcesses ( $sActiveMQHome,
                                               $sComponent,
                                               $sDXHome,
                                               $sHostname,
                                               $sInfaHome,
                                               $sMFTHome,
                                               $sUser ) ) {
    &logTrace ( 20029,
    "<%s> Processes for this instance do not exist after shutdown",
    $sMyName );
  }
  else {
    &logError ( 20030,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );

    $iReturnCode = 0; # Failure..
  }

  return $iReturnCode;
}



=pod

 -------------------------------------------------------------------------
 Function : bool Kill ( IN String ActiveMQHome,
                        IN Integer CleanReason,
                        IN String Component,
                        IN String DXHome,
                        IN String EnvFile,
                        IN String Hostname,
                        IN String InfaHome,
                        IN String MFTHome,
                        IN Integer Port,
                        IN String User );

 Description : Forces shutdown of resource needed for clean entry point

 Parameters :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $iCleanReason              Why clean was called
 $sComponent                Component for which the resource should be configured
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sEnvFile                  Absolute path to environment file to source
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $iPort                     Port to be used by the resource for startup, monitor, etc.
 $sUser                     System user credentials used to start the program
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Kill {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $iCleanReason,
       $sComponent,
       $sDXHome,
       $sEnvFile,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $iPort,
       $sUser ) = @_;

  #-----------------------------
  # Figure out why were called..
  #-----------------------------
  my ( $sCleanReason, %hsCleanReasons ) = ();
  my @lsCleanStrings = qw ( Offline_timed_out    Offline_was_ineffective
                            Online_timed_out      Online_was_ineffective
                            Found_offline_unexpectedly Monitor_timed_out );

  @hsCleanReasons { 0 .. $#lsCleanStrings } = @lsCleanStrings; # Populate..
  ( $sCleanReason = $hsCleanReasons { $iCleanReason } ||       # Reason..
                    $iCleanReason ) =~ s/_/ /g;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20031, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  CleanReason              [%s]
  Component                [%s]
  DXHome                   [%s]
  EnvFile                  [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  Port                     [%s]
  User                     [%s]",

  $sMyName, $sActiveMQHome, $sCleanReason,  $sComponent,  $sDXHome,
  $sEnvFile, $sHostname, $sInfaHome, $sMFTHome, $iPort, $sUser );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;
  
  #-----------------------------------------
  # Set the file name to save command output
  #-----------------------------------------
  my $sResourceType   = $ENV{'VCS_LOG_AGENT_NAME'};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};

  my $sOutputFilename    = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.out";
  
  if (-f "$sOutputFilename") {
    if (0 == unlink ("$sOutputFilename") ) {
      &logError ( 20169,
        "<%s> Unable to delete output file [%s]",
        $sMyName, $sOutputFilename );
      return 0;
    }
  }

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 ); # With margin..

  #--------------------------
  # Declare local variables..
  #--------------------------
  my ( $sCommand, $sCommandArgs, $iTimeout, $iExitCode,
       @lsFiles, @lsCmdout ) = ();
  
  #-------------------------------------
  # Define clean command and arguments..
  #-------------------------------------
  if ($sComponent eq "activemq") {
    $sCommand = "$sDXHome/bin/activemq.sh";
    $sCommandArgs = "stop";
  } elsif ($sComponent eq "dxconsole") {
    $sCommand = "$sDXHome/bin/dxconsole.sh";
    $sCommandArgs = "stop";
  } elsif ($sComponent eq "dxserver") {
    $sCommand = "$sDXHome/bin/dxserver.sh";
    $sCommandArgs = "stop $sHostname $iPort";
  } elsif ($sComponent eq "mft") {
    $sCommand = "$sDXHome/bin/mft.sh";
    $sCommandArgs = "stop";
  } elsif ($sComponent eq "powercenter") {
    $sCommand = "$sInfaHome/server/tomcat/bin/infaservice.sh";
    $sCommandArgs = "shutdown";
  }

  unless ( $hsCleanReasons { $iCleanReason } eq 'Offline_timed_out' ||
           $hsCleanReasons { $iCleanReason } eq 'Offline_was_ineffective' ) {
    #-------------------
    # Try to be gentle..
    #-------------------
    &logInfo ( 20032,
    "<%s> Attempting to stop Informatica with Command [%s] Args [%s]",
    $sMyName, $sCommand, $sCommandArgs );
  
    #------------------------
    # Request shutdown..
    #------------------------
    if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
      &logError ( 20033,
      "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
      $sMyName, abs ( $iTimeout ) );

      $iReturnCode = 0; # Failure..
    }
    elsif ( Informatica::ExecLocalCmd ( $sUser, 
                                        $sEnvFile, 
                                        $sCommand, 
                                        $sCommandArgs, 
                                        \$iExitCode, 
                                        $iTimeout, 
                                        \@lsCmdout, 
                                        $sOutputFilename ) ) {
      &logTrace ( 20034, 
      "<%s> Cmd [%s] with args [%s] executed successfully",
      $sMyName, $sCommand, $sCommandArgs );
    }
  }

  #-----------------------------------------
  # Whack any persisting Informatica processes..
  #-----------------------------------------
  &logTrace ( 20035,
  "<%s> Killing Informatica processes for this instance", $sMyName );

  if ( Informatica::KillInformaticaProcesses ( $sActiveMQHome,
                                               $sComponent,
                                               $sDXHome,
                                               $sHostname,
                                               $sInfaHome,
                                               $sMFTHome,
                                               $sUser ) ) {
    &logTrace ( 20036,
    "<%s> Processes for this instance do not exist after shutdown",
    $sMyName );
  }
  else {
    &logError ( 20037,
    "<%s> Could not remove all processes for this instance. Exiting",
    $sMyName );

    $iReturnCode = 0; # Failure..
  }
  
  
  return $iReturnCode;
}



=pod

 -------------------------------------------------------------------------
 Function : int GetResourceState ( IN String ActiveMQHome,
                                   IN String DXHome,
                                   IN String Hostname,
                                   IN String InfaHome,
                                   IN String MFTHome,
                                   IN String User );

 Description : First level monitor to determine state of resource.

 Parameters :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $sUser                     System user credentials used to start the program
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetResourceState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $sUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20038, "Subroutine <%s> called:
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  User                     [%s]",

  $sMyName,  $sActiveMQHome, $sComponent, $sDXHome,   $sHostname,   $sInfaHome,
  $sMFTHome, $sUser );
  
  #------------------------------------------
  # Check if virtual hostname is up locally..
  #------------------------------------------
  unless ( VCSagentFW::RunGetProcState ( $sHostname ) ) {
    &logTrace ( 20039,
    "<%s> Virtual Host [%s] not up locally", $sMyName, $sHostname );
    return VCSDef_MonitorExit::OFFLINE;
  }

  my @liDummyPids = ();
  my $iReturnCode = Informatica::GetProcState ( \@liDummyPids,
                                                $sActiveMQHome,
                                                $sComponent,
                                                $sDXHome,
                                                $sHostname,
                                                $sInfaHome,
                                                $sMFTHome,
                                                $sUser,
                                                1 ); # Use FFLM..

  if ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    &logTrace ( 20040,
    "<%s> Found all required processes. Return code set to [%s]",
    $sMyName, $iReturnCode );
  }
  else {
    &logTrace ( 20041,
    "<%s> Did not find all required processes. Returning [%s]",
    $sMyName, $iReturnCode );
  }

  #--------------------------------
  # Populate the global PID array..
  #--------------------------------
  @liRunningProcs = @liDummyPids;
                                                                                
  &logTrace ( 20042,
  "<%s> Populating global running proclist with [%s] PIDs [%s]",
  $sMyName, scalar @liRunningProcs, join ( ' ', @liRunningProcs ) );

  return ( $iReturnCode );
}


=pod

 -------------------------------------------------------------------------
 Function : int GetProcState ( OUT ListReference OnlinePids,
                               IN String ActiveMQHome,
                               IN String DXHome,
                               IN String Hostname,
                               IN String InfaHome,
                               IN String MFTHome,
                               IN String User,
                               IN Integer UseFFLM );

 Description : First level monitor to determine state of resource.

 Parameters :

 $rliOnlinePids             PIDs of relevant running processes
 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $sUser                     System user credentials used to start the program
 $iUseFFLM                  Filtering procedure
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetProcState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $rliOnlinePids,
       $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $sUser,
       $iUseFFLM ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20043, "Subroutine <%s> called with:
  OnlinePids               [%s]
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  User                     [%s]
  UseFFLM                  [%s]",

  $sMyName, $rliOnlinePids, $sActiveMQHome, $sComponent, $sDXHome,  $sHostname,
  $sInfaHome, $sMFTHome, $sUser, $iUseFFLM );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #----------------------
  # Initialise Variables..
  #----------------------
  my ( $iProcessCount, $iUseRFLM, $iFound, $iUid, $iPid, $sLang,
       $sKey, $rhlProc, $sPidFile, $sProc, $sTemp,
       @hProcListHash, @lsUpdates, @lsValues, %hiPids,
       %hiProcPids, %hsSrvrDetails, %hsCopy ) = ();

  @$rliOnlinePids = ();
  

  #------------------------------------
  # Attempt to get monitoring details..
  #------------------------------------
  if ( ( 0 == ( $iUid = getpwnam $sUser ) ) && # User is the super-user..
       ( 2 == $iUseFFLM ) ) {                  # And called to kill-by-UID..
    &logTrace ( 20044, "<%s> Unsafe to kill processes by User [%s] UID [%s].
    Using filter-based approach.", $sMyName, $sUser, $iUid ),
    $iUseFFLM = 1;
  }

  if ( $iUseFFLM != 2 ) { # Called for first level check..
    if ( Informatica::GetSrvrDetails ( $sActiveMQHome, 
                                       $sDXHome, 
                                       $sHostname, 
                                       $sInfaHome, 
                                       $sMFTHome, 
                                       $sComponent,
                                       \%hsSrvrDetails ) ) {
      &logTrace ( 20045,
      "<%s> Got details for monitoring Informatica instance", $sMyName );
  
      %hsCopy = %hsSrvrDetails;        # Backup..
      $iUseRFLM = scalar keys %hsCopy; # Set to entire count..
    }
    else {
      &logError ( 20046,
      "<%s> Failed to get details for monitoring Informatica instance",
      $sMyName );
      return VCSDef_MonitorExit::UNKNOWN;
    }
                                  
    #---------------------------------
    # Check if we attempt to be fast.. 
    #---------------------------------
    if ( $iUseFFLM ) { # FFLM..
      $iUseRFLM = 0;   # Hopefully skip..

      foreach $sKey ( keys %hsCopy ) {          # Use copy..
        @lsUpdates = ();                        # Reset..
        @lsValues = @{ $hsCopy { $sKey } };     # Get values..
        $sTemp .= "\nInformatica process [$sKey]\n"; # Display string..

        #----------------
        # Read in pairs..
        #----------------
        while ( ( $sPidFile, $sProc ) = splice ( @lsValues, 0, 2 ) ) {

          #-----------
          # Run FFLM..
          #-----------
          $iFound = Informatica::FFLMStatus ( $sPidFile,
                                         $sProc,
                                         $iUid,
                                         \$iPid );

          #----------------
          # Update status..
          #----------------
          push ( @lsUpdates, $sPidFile, $sProc, $iFound );

          $sTemp .= "PID File     [$sPidFile]\n";
          $sTemp .= "Srch Process [$sProc]\n";
          $sTemp .= "FFLM Status  [$iFound]\n\n";
        }

        @{ $hsCopy { $sKey } } = @lsUpdates;

        #------------------------
        # Handle monitor status..
        #------------------------
        if ( VCSDef_MonitorExit::ONLINE == $iFound ) {
          push ( @$rliOnlinePids, $iPid ); # Push..
          
        }
        else {
          ++ $iUseRFLM; # Mark it for RFLM..
        }
      }

      #----------------
      # Print summary..
      #----------------
      &logTrace ( 20047,
      "<%s> Informatica instance process details ---%s\n\n",
      $sMyName, $sTemp );

      ( $sTemp, $sPidFile, $sProc ) = (); # Release memory..
    }
    else { # Use RLFM..
      #-----------------------------------------
      # Mark status of each service as offline..
      #-----------------------------------------
      $iFound = VCSDef_MonitorExit::OFFLINE; # Set to offline..
      foreach $sKey ( keys %hsCopy ) {
        @lsUpdates = (); # Reset..

        @lsValues = @{ $hsCopy { $sKey } }; # Get values..
        while ( scalar @lsValues ) {
          push ( @lsUpdates, splice ( @lsValues, 0, 2 ), $iFound );
        }

        @{ $hsCopy { $sKey } } = @lsUpdates; # Update..
      }
    }
  }
  else { # Called for Kill..
    &logTrace ( 20048,
    "<%s> Called to get processes for User [%s] UID [%s]",
    $sMyName, $sUser, $iUid );
    ++ $iUseRFLM;
  }

  #-----------------------
  # Regular slow :-( FLM..
  #-----------------------
  if ( $iUseRFLM ) {
    &logTrace ( 20049,
    "<%s> Running regular first level monitor", $sMyName );

    #----------------------------------
    # Set LANG for better performance..
    #----------------------------------
    $sLang = $ENV { LANG }; # Backup..
    $ENV { LANG } = 'C';    # Now set..

    #-----------------------------------
    # Call processes and store in hash..
    #-----------------------------------
    $iProcessCount = Proc::GetLongProcessListHash ( \@hProcListHash );

    #----------
    # Restore..
    #----------
    defined $sLang ? $ENV { LANG } = $sLang : delete $ENV { LANG }; # Restore..

    #----------------------
    # Check if we got any..
    #----------------------
    if ( $iProcessCount ) {
      &logTrace ( 20050,
      "<%s> Got [%s] processes from proc table for filtering",
      $sMyName, $iProcessCount );
    }
    else {
      &logError ( 20051,
      "<%s> Call to get process list did not return any processes",
      $sMyName );
      return VCSDef_MonitorExit::OFFLINE; # Offline..
    }

    #-----------------------
    # Search for processes..
    #-----------------------
    foreach $rhlProc ( @hProcListHash ) {
      #--------------------------------------------------------------
      # Each $rhlProc is a ref. to a hash with the following keys..
      # "UID"     =>  <user id>
      # "PID"     =>  <process id>
      # "PPID"    =>  <parent process id>
      # "COMMAND" =>  <command string>
      #--------------------------------------------------------------

      next unless defined $$rhlProc { UID }; # Bug in Proc::GetLongProcessListHash..
      next if $$rhlProc { PID } == $$;       # Skip self..
      $iPid = $$rhlProc { PID };             # Assign..

      &logTrace ( 20052, "<%s> Searching UID [%s] PID [%s] PPID [%s] CMD [%s]",
      $sMyName, $$rhlProc { UID }, $$rhlProc { PID }, $$rhlProc { PPID },
      $$rhlProc { COMMAND } );

      if ( $$rhlProc { UID } == $iUid ) { # Process owner..
        #-----------------------------------------------
        # Look for processes that were offline in FFLM..
        #-----------------------------------------------
        if ( $iUseFFLM != 2 ) { # Request to filter..
          foreach $sKey ( keys %hsCopy ) {
            @lsUpdates = (); # Reset..
            @lsValues = @{ $hsCopy { $sKey } }; # Get values..
    
            #--------------------------------------------------
            # Should have details per three elements of array..
            # <PIDFILE> <PROCESS> <STATE>
            #--------------------------------------------------
            while ( ( $sPidFile, $sProc, $iFound ) =
                      splice ( @lsValues, 0, 3 ) ) {
              if ( VCSDef_MonitorExit::OFFLINE == $iFound ) { # Earlier marked offline..
                if ( $$rhlProc { COMMAND } =~ /$sProc/ ) {    # Now found..
                  unshift ( @$rliOnlinePids, $iPid );         # Store..
    
                  &logTrace ( 20053,
                  "<%s> Found PID [%s] PPID [%s] CMD [%s] with filter [%s]",
                  $sMyName, $iPid, $$rhlProc { PPID }, $$rhlProc { COMMAND },
                  $sProc );

                  $hiProcPids { $sKey } = $iPid;        # Store..
                  
                  $iFound = VCSDef_MonitorExit::ONLINE; # Mark online..
                }
              }
    
              #----------------
              # Update status..
              #----------------
              push ( @lsUpdates, $sPidFile, $sProc, $iFound );
            }
    
            @{ $hsCopy { $sKey } } = @lsUpdates; # Update..
          }
        }
        else { # Request for kill..
          &logTrace ( 20054,
          "<%s> Pushing PID [%s] with PPID [%s] Cmd [%s] on stack",
          $sMyName, $iPid, $$rhlProc { PPID }, $$rhlProc { COMMAND } );
          push ( @$rliOnlinePids, $iPid ); # Store..
        } 
      }
      
    }
  }

  #------------------------------------
  # Set final status for each process..
  #------------------------------------
  my ( $iOnline, $iOffline, $sEntryPointName ) = ();

  ( $sEntryPointName = $0 ) =~ s#.*/##; # Basename..
  $sTemp = ''; # Ensure..

  foreach $sKey ( keys %hsCopy ) {
    @lsValues = @{ $hsCopy { $sKey } }; # Get values..
    $sTemp .= "\nProcess      [$sKey]\n";

    while ( ( $sPidFile, $sProc, $iFound ) =
              splice ( @lsValues, 0, 3 ) ) {
      #-----------------------------------
      # Create PID files for use in FFLM..
      #-----------------------------------
      if ( $iUseRFLM                                 &&      # In RFLM..
           $iUseFFLM != 2                            &&      # Asked to filter..
           $hiProcPids { $sKey }                     &&      # PID populated..
           ( $sEntryPointName =~ /^(monitor|online)$/o ) ) { # In monitor or online..

        &logTrace ( 20055,
        "<%s> Creating PID file [%s] with PID [%s]",
        $sMyName, $sPidFile, $hiProcPids { $sKey } );

        #---------
        # Create..
        #---------
        if ( Informatica::CreatePidFile ( $sPidFile,
                                     $hiProcPids { $sKey } ) ) {
          &logTrace ( 20056,
          "<%s> Successfully created PID file [%s]", $sMyName, $sPidFile );
        }
        else {
          &logError ( 20057,
          "<%s> Could not create PID file [%s] successfully",
          $sMyName, $sPidFile );
        }
      }

      #-------------
      # Set status..
      #-------------
      if ( VCSDef_MonitorExit::ONLINE == $iFound ) {
        $iOnline = $iFound;
      }
      elsif ( VCSDef_MonitorExit::OFFLINE == $iFound ) {
        $iOffline = $iFound;
      }

      $sTemp .= "Srch Process [$sProc]\n";
      $sTemp .= "FLM Status   [$iFound]\n\n";
    }

    $sTemp .= "--------------------";
  }

  #-------------------
  # Set final status..
  #-------------------
  $iReturnCode = $iOffline || $iOnline || VCSDef_MonitorExit::UNKNOWN;

  #------------------------------------------------
  # Reset status based on PIDs, if not in monitor..
  #------------------------------------------------
  if ( $sEntryPointName ne 'monitor'                   &&
       ( VCSDef_MonitorExit::OFFLINE == $iReturnCode ) &&
       scalar @$rliOnlinePids ) {
    $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;

    &logTrace ( 20058,
    "<%s> Found [%s] PIDs [%s] for this instance of Informatica.
    Resetting return code to [%s]",
    $sMyName, scalar @$rliOnlinePids, join ( ' ', @$rliOnlinePids ),
    $iReturnCode );
  }

  #-----------------------
  # Report on FLM status..
  #-----------------------
  if ( $iUseFFLM != 2 ) { # Request to filter..
    &logTrace ( 20059,
    "<%s> Informatica process details ---\n%s\nReturn code is [%s]",
    $sMyName, $sTemp, $iReturnCode );
    $sTemp = undef; # Release..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int Monitor ( IN String ActiveMQHome,
                          IN String Component,
                          IN String DXHome,
                          IN String EnvFile,
                          IN String Hostname,
                          IN String InfaHome,
                          IN String MFTHome,
                          IN String MonitorProgram,
                          IN Integer Port,
                          IN Integer SecondLevelMonitor,
                          IN String User,
                          IN Integer $level_two );

 Description: Performs the in-depth monitoring

 Parameters :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sComponent                Component for which the resource should be configured
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sEnvFile                  Absolute path to environment file to source
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $sMonitorProgram           Full path to external monitor program
 $iPort                     Port to be used by the resource for startup, monitor, etc.
 $iSecondLevelMonitor       Cycle count to run in-depth monitoring
 $sUser                     System user credentials used to start the program
 $level_two                Level Two Monitoring when IMF Enabled
 
 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if in-depth monitoring cannot be completed.
 -------------------------------------------------------------------------

=cut

sub Monitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sEnvFile,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $sMonitorProgram,
       $iPort,
       $iSecondLevelMonitor,
       $sUser,
       $level_two ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20060, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  EnvFile                  [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  MonitorProgram           [%s]
  Port                     [%s]
  SecondLevelMonitor       [%s]
  User                     [%s]
  LevelTwoMonitor          [%s]",

  $sMyName,  $sActiveMQHome,  $sComponent,   $sDXHome,   $sEnvFile,
  $sHostname,  $sInfaHome,  $sMFTHome,  $sMonitorProgram,   $iPort,
  $iSecondLevelMonitor, $sUser, $level_two );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  my $iTillWhen = undef;

  #----------------------------------------------
  # In-depth monitoring, if correct time to run..
  #----------------------------------------------
  if ( -300 == $level_two ) {
     $level_two = VCSagentFW::RunSecondLevelMonitor ( $iSecondLevelMonitor );  
     &logTrace ( 20061, "<%s> Traditional second level monitoring, SLM flag for this monitor set to [%s] ", $sMyName, $level_two  );
  }

  if ( $level_two >= 1 ) 
  {
    #--------------------------
    # Just In Time Validation..
    #--------------------------
    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =
         Informatica::DelayedAttributeCheck ( $sActiveMQHome,
                                              $sComponent,
                                              $sDXHome,
                                              $sEnvFile,
                                              $sHostname,
                                              $sInfaHome,
                                              $sMFTHome,
                                              $iPort,
                                              $sUser ) ) ) {
      &logTrace ( 20062,
      "<%s> Delayed attribute check completed successfully",
      $sMyName );
    }
    else {
      &logError ( 20063,
      "<%s> Attribute validation failed. Returning [%s]",
      $sMyName, $iReturnCode );
      return $iReturnCode;
    }

    &logTrace ( 20064,
    "<%s> Proceeding with second level monitoring", $sMyName );

    #-------------------------------------------------------
    # Upper bound..
    # Time in EPOCH when entry pt started + VCS time limit..
    #-------------------------------------------------------
    $iTillWhen = $^T + VCSagentFW::GetEngineTimeout ( 1 );

    if ( VCSDef_MonitorExit::ONLINE == ( $iReturnCode =
         Informatica::SecondLevelMonitor ( $sActiveMQHome,
                                           $sComponent,
                                           $sDXHome,
                                           $sEnvFile,
                                           $sHostname,
                                           $sInfaHome,
                                           $sMFTHome,
                                           $iPort,
                                           $sUser,
                                           $iTillWhen ) ) ) {

      &logTrace ( 20065,
      "<%s> Informatica instance online with status [%s] after SLM",
      $sMyName, $iReturnCode );
    }
    else {
      &logError ( 20066,
      "<%s> Informatica instance NOT online after SLM. Status [%s]",
      $sMyName, $iReturnCode );
      return $iReturnCode;
    }
  }

  #------------------------------------------------
  # Exit if external monitor has not been defined..
  #------------------------------------------------
  return $iReturnCode unless defined $sMonitorProgram;

  #---------------------------------------
  # MonitorProgram defined..
  # Separate the executable and the args..
  #---------------------------------------
  my ( $sExtCmd, $sExtCmdArgs ) = $sMonitorProgram # Separate
                                =~ /(\S+)\s*(.*)/; # Cmd and args..

  $sExtCmdArgs = '' unless defined $sExtCmdArgs;   # Prevent undefs..

  if ( $sExtCmd ) {                                # Defined..
    #----------------------------------------
    # Get time bearings, if not got earlier..
    #----------------------------------------
    $iTillWhen ||= $^T + VCSagentFW::GetEngineTimeout ( 1 );

    if ( VCSDef_MonitorExit::ONLINE ==             # and found online..
         ( $iReturnCode =                          # by the status..
         Informatica::ExternalMonitor ( $sUser,
                                        $sEnvFile,
                                        $sExtCmd,
                                        $sExtCmdArgs,
                                        $iTillWhen ) ) ) {
      &logTrace ( 20067,
      "<%s> Informatica instance found online with status [%s]
      after executing external monitor [%s] with args [%s]",
      $sMyName, $iReturnCode, $sExtCmd, $sExtCmdArgs );
    }
    else {
      &logError ( 20068,
      "<%s> Informatica instance NOT online after executing external monitor
      [%s] with args [%s]. Status [%s]",
      $sMyName, $sExtCmd, $sExtCmdArgs, $iReturnCode );
    }
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int DelayedAttributeCheck ( IN String ActiveMQHome,
                                        IN String Component,
                                        IN String DXHome,
                                        IN String EnvFile,
                                        IN String Hostname,
                                        IN String InfaHome,
                                        IN String MFTHome,
                                        IN Integer Port,
                                        IN String User );

 Description : Just In Time Validation

 Parameters :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sComponent                Component for which the resource should be configured
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sEnvFile                  Absolute path to environment file to source
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $iPort                     Port to be used by the resource for startup, monitor, etc.
 $sUser                     System user credentials used to start the program
 
 Returns  : Integer value as defined below:

 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub DelayedAttributeCheck {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sEnvFile,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $iPort,
       $sUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20069, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  EnvFile                  [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  Port                     [%s]
  User                     [%s]",

  $sMyName,  $sActiveMQHome,  $sComponent,   $sDXHome,   $sEnvFile,
  $sHostname, $sInfaHome, $sMFTHome, $iPort, $sUser );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my @lsArgs = qw ( ActiveMQHome Component DXHome EnvFile Hostname InfaHome MFTHome Port User );
  my @lsReqdArgs = qw ( Component Hostname Port User );
  my ( @lsMissing, %hsArgList ) = ();

  @hsArgList { @lsArgs } = @_; # Assign..

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20070,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #--------------------
  # Return, if failed..
  #--------------------
  if ( VCSDef_MonitorExit::ONLINE != $iReturnCode ) {
    return $iReturnCode;
  }

  #---------------------------
  # Validate parameter User..
  #---------------------------
  if ( ! defined getpwnam $sUser ) {
    &logError ( 20071,
    "<%s> User [%s] is not a valid UNIX login",
    $sMyName, $sUser );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #-------------------------------
  # Validate parameter Hostname..
  #-------------------------------
  if ( ! inet_aton $sHostname ) {
    &logError ( 20072,
    "<%s> Hostname [%s] is an invalid Hostname/IPv4 address",
    $sMyName, $sHostname );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #------------------------------
  # Validate parameter EnvFile..
  #------------------------------
  if ( $sEnvFile ) {
    if ( ! -e $sEnvFile ) {
      &logError ( 20073, "<%s> EnvFile [%s] file does not exist",
        $sMyName, $sEnvFile );
      $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
    }
  }

  #---------------------------
  # Validate parameter Port..
  #---------------------------
  if ( ! $iPort ) {
      &logError ( 20074, "<%s> Port [%s] has to be greater than 0",
          $sMyName, $iPort );
      $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }

  #-----------------------------------
  # Validate parameter ActiveMQHome..
  #-----------------------------------
  if ($sComponent eq "activemq") {
      if ( ! -d $sActiveMQHome ) {
          &logError ( 20075,
              "<%s> ActiveMQHome [%s] directory does not exist",
          $sMyName, $sActiveMQHome );
          $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
      }
  }

  #-----------------------------
  # Validate parameter DXHome..
  #-----------------------------
  if ( $sComponent eq "activemq" ||
       $sComponent eq "dxconsole" ||
       $sComponent eq "dxserver" ||
       $sComponent eq "mft" ) {
       if ( ! -d $sDXHome ) {
          &logError ( 20076,
              "<%s> DXHome [%s] directory does not exist",
              $sMyName, $sDXHome );
          $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
      }
  }

  #-------------------------------
  # Validate parameter InfaHome..
  #-------------------------------
  if ( $sComponent eq "informatica" ) {
      if ( ! -d $sInfaHome ) {
          &logError ( 20077,
              "<%s> InfaHome [%s] directory does not exist",
              $sMyName, $sInfaHome );
          $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
      }
  }

  #------------------------------
  # Validate parameter MFTHome..
  #------------------------------
  if ( $sComponent eq "mft" ) {
      if ( ! -d $sMFTHome ) {
          &logError ( 20078,
              "<%s> MFTHome [%s] directory does not exist",
              $sMyName, $sMFTHome );
          $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
      }
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int SecondLevelMonitor ( IN String ActiveMQHome,
                                     IN String Component,
                                     IN String DXHome,
                                     IN String EnvFile,
                                     IN String Hostname,
                                     IN String InfaHome,
                                     IN String MFTHome,
                                     IN Integer Port,
                                     IN String User,
                                     IN Integer TillWhen );

 Description : Does in-depth monitoring, if enabled.

 Parameters :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sComponent                Component for which the resource should be configured
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sEnvFile                  Absolute path to environment file to source
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $iPort                     Port to be used by the resource for startup, monitor, etc.
 $sUser                     System user credentials used to start the program
 $iTillWhen                 Time in EPOCH left for IInd level monitoring
 
 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if the command cannot be completed.
 -------------------------------------------------------------------------

=cut

sub SecondLevelMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sEnvFile,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $iPort,
       $sUser,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20079, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  EnvFile                  [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  Port                     [%s]
  User                     [%s]
  TillWhen                 [%s]",

  $sMyName,  $sActiveMQHome,  $sComponent,   $sDXHome,   $sEnvFile,
  $sHostname, $sInfaHome, $sMFTHome, $iPort, $sUser, $iTillWhen );  

  #------------------
  # Hopeful success..
  #------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;
  
  #-----------------------------------------
  # Set the file name to save command output
  #-----------------------------------------
  my $sResourceType   = $ENV{'VCS_LOG_AGENT_NAME'};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};
  my $sEntryPointName = $ENV{'VCS_LOG_SCRIPT_NAME'};

  my $sOutputFilename    = "/tmp/.VRTS$sResourceType/$sResourceName"."_$sEntryPointName.slm.out";
  
  if (-f "$sOutputFilename") {
    if (0 == unlink ("$sOutputFilename") ) {
      &logError ( 20170,
        "<%s> Unable to delete output file [%s]",
        $sMyName, $sOutputFilename );
      return 0;
    }
  }

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iTimeout, $iExitCode, $sCommand, $sCommandArgs, @lsCmdout ) = ();
  
  #------------------------------------
  # Check if port can be connected to..
  #------------------------------------
  if ( Comms::CheckPort ( $sHostname,
                          $iPort ) ) {
    &logTrace ( 20080,
    "<%s> Server listening on Host [%s] Port [%s]: Socket method",
    $sMyName, $sHostname, $iPort );
  }
  else {
    &logError ( 20081,
    "<%s> Server NOT listening on Host [%s] Port [%s]: Socket method",
    $sMyName, $sHostname, $iPort );

    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  #------------------------
  # Return, unless online..
  #------------------------
  unless ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    return $iReturnCode;
  }

  #---------------------------------------------
  # Declare monitor command and args variables..
  #---------------------------------------------

  if ($sComponent eq "activemq") {
	if ( -f "$sActiveMQHome/bin/activemq-admin" ) {
		$sCommand = "$sActiveMQHome/bin/activemq-admin";
	} elsif ( -f "$sActiveMQHome/bin/activemq" ) {
		$sCommand = "$sActiveMQHome/bin/activemq";
	} else {
		&logError ( 20173, "<%s>: activemq-admin binary not found.", $sMyName);
		return VCSDef_MonitorExit::OFFLINE;
    	}
	$sCommandArgs = "list --jmxurl service:jmx:rmi:///jndi/rmi://localhost:$iPort/jmxrmi";

  } elsif ( $sComponent eq "dxconsole" ||
            $sComponent eq "powercenter" ) {
    #------------------------------------
    # Check if port can be connected to..
    #------------------------------------
    my $iHTTPRet = Informatica::HTTPConnect ( $sHostname,$iPort, "" );
    if ( $iHTTPRet == 200 ) {
      # maybe added 302 as well ?!?
      &logTrace ( 20082, "<%s>: Connection to [%s]:[%s] for [%s] successful. Return Code: [%d]",
             $sMyName, $sHostname, $iPort, $sComponent, $iHTTPRet );
      $iReturnCode = VCSDef_MonitorExit::ONLINE;
    } else {
      &logTrace ( 20083, "<%s>: Connection to [%s]:[%s] for [%s] not responding. Return Code: [%d]",
             $sMyName,$sHostname,$iPort, $sComponent, $iHTTPRet );
      $iReturnCode = VCSDef_MonitorExit::OFFLINE;
    }

    #------------------------------------------------------------
    # Return immediately as there is no command execution for SLM
    #------------------------------------------------------------
    return $iReturnCode;

  } elsif ($sComponent eq "dxserver") {
    $sCommand = "$sDXHome/bin/dxserver.sh";
    $sCommandArgs = "ping $sHostname $iPort";

  } elsif ($sComponent eq "mft") {

      # INFO: "port connect" has already been performed.
      return VCSDef_MonitorExit::ONLINE;
  }

  #-------------------------------
  # Validate and execute command..
  #-------------------------------
  if ( ( $iTimeout = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20084,
    "<%s> Insufficient time to continue in-depth monitoring.
    Short by [%s] sec",
    $sMyName, abs ( $iTimeout ) );

    $iReturnCode = VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  elsif ( Informatica::ExecLocalCmd ( $sUser, 
                                      $sEnvFile, 
                                      $sCommand, 
                                      $sCommandArgs, 
                                      \$iExitCode, 
                                      $iTimeout, 
                                      \@lsCmdout, 
                                      undef ) ) {
    &logTrace ( 20085,
    "<%s> Cmd [%s] with args [%s] executed successfully",
    $sMyName, $sCommand, $sCommandArgs );

    my $bSearchOutput = 0;
    my $sSrchString = "";

    if ($sComponent eq "activemq") {
      $sSrchString = "BrokerName = localhost";
      $bSearchOutput = 1;
    } elsif ($sComponent eq "dxconsole") {
      # nothing, fall through ...
    } elsif ($sComponent eq "dxserver") {
      $sSrchString = "* Received Ping from rmi://$sHostname:$iPort/DXRuntimeControl - replied all is well *";
      $bSearchOutput = 1;
    } elsif ($sComponent eq "mft") {
      # nothing, fall through ...
    } elsif ($sComponent eq "powercenter") {
      # nothing, fall through ...
    }

    if ($bSearchOutput) {
      if ( grep (/\Q$sSrchString/i, @lsCmdout) ) {
        &logTrace ( 20086,
        "<%s> Component [%s]: Expected string [%s] found in command output",
        $sMyName, $sComponent, $sSrchString);
      } else {
        &logError ( 20087,
        "<%s> Component [%s]: Expected string [%s] not found in command output",
        $sMyName, $sComponent, $sSrchString);

        $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
      }
    }
  }
  elsif ( ! defined $iExitCode ) {
    $lsCmdout [0] ||= 'undef'; # Set if not defined..
    $iReturnCode = $lsCmdout [0] eq 'timeout'? VCSDef_MonitorExit::TIMEOUT: # Timeout..
                                               VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  else {
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int ExternalMonitor ( IN String User,
                                  IN String EnvFile,
                                  IN String Cmd,
                                  IN String CmdArgs,
                                  IN Integer TillWhen );

 Description : Handles user defined external monitor program

 Parameters :

 $sUser                     System user credentials used to start the program
 $sEnvFile                  Absolute path to environment file to source
 $sCmd                      User defined utilty to execute
 $sCmdArgs                  Command line arguments
 $iTillWhen                 Time in EPOCH left for IInd level monitoring
 
 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 ??   HUNG: External monitor timed out         VCSDef_MonitorExit::TIMEOUT

 The entry point hangs if the command cannot be completed.
 -------------------------------------------------------------------------

=cut

sub ExternalMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sUser,
       $sEnvFile,
       $sCmd,
       $sCmdArgs,
       $iTillWhen ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20088, "Subroutine <%s> called with:
  User                     [%s]
  EnvFile                  [%s]
  Cmd                      [%s]
  CmdArgs                  [%s]
  TillWhen                 [%s]",

  $sMyName, $sUser, $sEnvFile, $sCmd, $sCmdArgs, $iTillWhen );

  #------------------
  # Hopeful success..
  #------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  #------------------------
  # Setup local variables..
  #------------------------
  my ( $iExitCode, $iTimeout, @lsCmdout ) = ();
  my $iBuf = 5; # Secs..

  #-------------------------
  # Handled exit code hash..
  #-------------------------
  my %hiExitCodes = (  0, VCSDef_MonitorExit::ONLINE,
                     110, VCSDef_MonitorExit::ONLINE,
                       1, VCSDef_MonitorExit::OFFLINE,
                     100, VCSDef_MonitorExit::OFFLINE,
                          VCSDef_MonitorExit::TIMEOUT,
                          VCSDef_MonitorExit::TIMEOUT );

  #---------------------------------------------
  # Check if command can be found and executed..
  #---------------------------------------------
  if ( ! -e $sCmd ) {
   &logError ( 20089,
    "<%s> External monitor [%s] does not exist",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( ! -x $sCmd ) {
   &logError ( 20090,
    "<%s> External monitor [%s] not executable.",
    $sMyName, $sCmd );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  elsif ( $sEnvFile =~ /\S/ && ! -e $sEnvFile ) {
    &logError ( 20091,
    "<%s> EnvFile [%s] does not exist", $sMyName, $sEnvFile );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
elsif ( ( $iTimeout = $iTillWhen - ( time + $iBuf ) ) <= 5 ) { # Lack of time..
   &logError ( 20092,
    "<%s> Insufficient time to proceed with external monitor [%s].
    Short by [%s] sec", $sMyName, $sCmd, abs ( $iTimeout ) );
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }

  #--------------------------------------------
  # Return if validation failed at this point..
  #--------------------------------------------
  unless ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    return $iReturnCode;
  }

  &logTrace ( 20093,
  "<%s> Executing external monitor with Command [%s] Args [%s] in [%s] secs",
  $sMyName, $sCmd, $sCmdArgs, $iTimeout );

  #----------------------
  # Execute the command..
  #----------------------
  Informatica::ExecLocalCmd ( $sUser, 
                              $sEnvFile, 
                              $sCmd, 
                              $sCmdArgs, 
                              \$iExitCode, 
                              $iTimeout, 
                              \@lsCmdout );
                               
  #------------------------
  # Report on exit status..
  #------------------------
  if ( ! defined $iExitCode ) {
    $lsCmdout [0] ||= 'undef'; # Set if not defined..
    $iExitCode = $lsCmdout [0] eq 'timeout'? VCSDef_MonitorExit::TIMEOUT: # Timeout..
                                             VCSDef_MonitorExit::UNKNOWN; # Failure..
  }
  elsif ( $hiExitCodes { $iExitCode } &&
          VCSDef_MonitorExit::ONLINE ==
          $hiExitCodes { $iExitCode } ) {
    &logTrace ( 20094,
    "<%s> Command [%s] with args [%s] executed successfully with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdout ) );
  }
  else {
    &logWarn ( 20095,
    "<%s> Command [%s] with args [%s] execution failed with exit code [%s].
    Cmd output [%s]",
    $sMyName, $sCmd, $sCmdArgs, $iExitCode, join ( chr 012, @lsCmdout ) );
  }

  #--------------------------------
  # Setup appropriate return code..
  #--------------------------------
  $iReturnCode = $hiExitCodes { $iExitCode } ||
                 VCSDef_MonitorExit::UNKNOWN;

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool CreatePidFile ( IN String PidFile,
                                 In Integer Pid );


 Description : Creates PID file with given PID

 Parameters :

 $sPidFile     File to create
 $iPid         PID to store

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub CreatePidFile {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sPidFile,
       $iPid ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20096, "Subroutine <%s> called with:
  PidFile         [%s]
  Pid             [%s]",

  $sMyName, $sPidFile, $iPid );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-----------------------
  # Check against undefs..
  #-----------------------
  unless ( defined $iPid && $iPid =~ /^\d+$/o) {
    $iPid ||= 'undef'; # Prevent undefs.
    &logError ( 20097, "<%s> PID [%s] is not a valid positive number",
    $sMyName, $iPid );
    return 0; # Proceed no further..
  }

  #------------------
  # Create PID file..
  #------------------
  &logTrace ( 20098,
  "<%s> Creating PID file [%s] with PID [%s]",
  $sMyName, $sPidFile, $iPid );

  if ( sysopen ( PIDFILE, $sPidFile, O_WRONLY|O_SYNC|O_CREAT|O_TRUNC,
                 0600 ) ) {
    print PIDFILE $iPid, "\n"; # Update..

    unless ( close ( PIDFILE ) ) {
      &logWarn ( 20099,
      "<%s> Failed to close PidFile [%s] after update: [%s]",
      $sMyName, $sPidFile, $! );
      $iReturnCode = 0;
    }
  }
  else {
    &logWarn ( 20100,
    "<%s> Failed to create PID file [%s] with PID [%s]",
    $sMyName, $sPidFile, $iPid );
    $iReturnCode = 0;
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int FFLMStatus ( IN String PidFile,
                             IN String Proc,
                             IN Integer Uid,
                             OUT RefInt Pid );

 Description: Checks status of Process listed in PidFile, if running

 Parameters :

 $sPidFile       Absolute path to PID file of service
 $sProc          Process string to search for in proc table
 $iUid           UNIX UID of owner
 $riPid          Reference to existing PID

 Returns : Integer value as defined below:
 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 -------------------------------------------------------------------------

=cut

sub FFLMStatus {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sPidFile,
       $sProc,
       $iUid,
       $riPid ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20101, "Subroutine <%s> called with:
  PidFile                [%s]
  Proc                   [%s]
  UID                    [%s]
  riPid                  [%s]",

  $sMyName, $sPidFile, $sProc, $iUid, $riPid );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..
  $$riPid = 0; # Reset..

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $iPid, $rhlProc, @lProcessHash ) = ();

  #----------------------------------
  # Start by testing the attributes..
  #----------------------------------
  if ( $sProc !~ /\S/ ) {
    &logError ( 20102, "<%s> Called with empty [%s] process string",
    $sMyName, $sProc );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }
  elsif ( ! -e $sPidFile ) {
    &logTrace ( 20103, "<%s> PID file [%s] does not exist",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }
  elsif ( ! -s $sPidFile ) {
    &logWarn ( 20104, "<%s> PID file [%s] is of zero bytes",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  #---------------------
  # Return, if offline..
  #---------------------
  return $iReturnCode unless VCSDef_MonitorExit::ONLINE == $iReturnCode;
 
  if ( open ( PIDFILE, $sPidFile ) ) {
    ( $iPid ) = grep { ( s/\n// || 1 ) && # Chomp..
                         s/.*?(\d+)$/$1/  # Get PID..
                } <PIDFILE>;

    $iPid = '' unless defined $iPid;      # Avoid undefs..

    close ( PIDFILE ) ||                  # Close file..
    &logWarn ( 20105,
    "<%s> Could not close PID file [%s] after read [%s]",
    $sMyName, $sPidFile, $! );
  }
  else {
    &logError ( 20106,
    "<%s> Failed to open PID file [%s] for read: [%s]",
    $sMyName, $sPidFile, $! );
    return VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  #------------------------
  # Check if PID is valid..
  #------------------------
  if ( $iPid =~ /\d+/ &&     # If the valid number..
       kill ( 0, $iPid ) ) { # is a PID that's alive..

    &logTrace ( 20107,
    "<%s> PID [%s] from PID file [%s] exists",
    $sMyName, $iPid, $sPidFile );

    #-----------------------
    # Search in proc table..
    #-----------------------
    if ( Informatica::GetPidDetails ( $iPid,
                                 \@lProcessHash ) ) {
      #--------------
      # Got details..
      #--------------
      foreach $rhlProc ( @lProcessHash ) {
        &logTrace ( 20108,
        "<%s> Process details: UID [%s] PID [%s] PPID [%s] ARGS [%s]",
        $sMyName, $$rhlProc { UID }, $$rhlProc { PID }, $$rhlProc { PPID },
        $$rhlProc { COMMAND } );

        #--------------
        # Now compare..
        #--------------
        if ( ( $$rhlProc { UID } == $iUid ) &&       # Owner match..
               $$rhlProc { COMMAND } =~ /$sProc/ ) { # Filter match..
          &logTrace ( 20109,
          "<%s> PID [%s] owned by UID [%s] with ARGS [%s] contains [%s]",
          $sMyName, $iPid, $$rhlProc { UID }, $$rhlProc { COMMAND }, $sProc );
          $$riPid = $iPid; # Store..
        }
        else {
          &logTrace ( 20110,
          "<%s> Ignoring PID [%s] owned by UID [%s] with ARGS [%s] with string [%s]",
          $sMyName, $iPid, $$rhlProc { UID }, $$rhlProc { COMMAND }, $sProc );
          $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
        }
      }
    }
    else {
      &logTrace ( 20111,
      "<%s> Could not get details for PID [%s]",
      $sMyName, $iPid );
      $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
    }
  }
  else {
    &logTrace ( 20112, "<%s> PID [$iPid] does not exist",
    $sMyName, $sPidFile );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE; # Failure..
  }

  @lProcessHash = (); # Truncate..
  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool KillInformaticaProcesses ( IN String ActiveMQHome,
                                            IN String Component,
                                            IN String DXHome,
                                            IN String Hostname,
                                            IN String InfaHome,
                                            IN String MFTHome,
                                            IN String User );

 Description  : Kills processes for this instance of Informatica
 
 Parameters :

 $sActiveMQHome           Absolute path to the ActiveMQ component's installation directory
 $sComponent              Component for which this resource has to be configured
 $sDXHome                 Absolute path to the B2B Data Exchange installation directory
 $sHostname               Hostname on which the resource is configured
 $sInfaHome               Absolute path to the PowerCenter installation directory.
 $sMFTHome                Absolute path to the Managed File Transfer component's installation directory.
 $sUser                   System user credentials used to start the program
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------
=cut

sub KillInformaticaProcesses {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sComponent,
       $sDXHome,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $sUser ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20113, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  Component                [%s]
  DXHome                   [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  User                     [%s]",

  $sMyName,  $sActiveMQHome, $sComponent, $sDXHome,   $sHostname,   $sInfaHome,
  $sMFTHome, $sUser );
 
  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1; # Success..

  #-------------------------
  # Set up local variables..
  #-------------------------
  my ( @liGoners, @liUnDead ) = ();

  #---------------------------------------------
  # Get list of processes running for this Uid..
  #---------------------------------------------
  if ( ( caller 1 )[3] eq 'Informatica::Start' ) { # During online..
    @liGoners = @liRunningProcs;              # Get PIDs to kill..
  }
  else {
    Informatica::GetProcState ( \@liGoners,
                                $sActiveMQHome,
                                $sComponent,
                                $sDXHome,
                                $sHostname,
                                $sInfaHome,
                                $sMFTHome,
                                $sUser,
                                0 ); # Set filtering choice..
  }

  #--------------------------
  # Sort in ascending order..
  #--------------------------
  @liGoners = sort { $a <=> $b } @liGoners;

  #------------------------------------------------------------------
  # We do a two level kill. The SIGINQ ensures  that  the  PID  is  a
  # valid process before we actually issue the SIGKILL. We don't want
  # to report a failure for SIGKILL in case  of  defunct  or  missing
  # processes.
  #------------------------------------------------------------------
  if ( scalar @liGoners ) {
    #----------------------
    # Failed to kill some..
    #----------------------
    if ( @liUnDead = grep {
                         kill ( 0, $_ ) && # Valid..
                       ! kill ( 9, $_ )    # and could NOT be killed..
                     } @liGoners ) {       # From what's found..
      &logError ( 20114,
      "<%s> PIDs not killed [%s] Total not killed [%s]",
      $sMyName, join ( ' ', @liUnDead ), scalar @liUnDead );
      $iReturnCode = 0; # Failure..
    }
    else { # All killed..
      &logInfo ( 20115,
      "<%s> Killed PIDs [%s] Total killed: [%s]",
      $sMyName, join ( ' ', @liGoners ), scalar @liGoners );
    }
  }
  else {
    &logTrace ( 20116,
    "<%s> Could not locate any process this instance of Informatica",
    $sMyName );
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool ExecLocalCmd ( IN String User,
                                IN String EnvFile,
                                IN String Command,
                                IN String CommandArgs,
                                OUT RefInt ExitCode,
                                IN Integer Timeout,
                                OUT ListReference Cmdout );

 Description : Wrapper function for
               Sys::RunWithEnvCmdWithOutputWithTimeOut to collect output
               of the command.
 
 Parameters :

 $sUser            UNIX user to run the command as
 $sEnvFile         Absolute path to location of EnvFile to source
 $sCommand         Absolute path to location of command
 $sCommandArgs     Arguments to command
 $riExitCode       Reference to collect exit code of command
 $iTimeout         Time limit to execute the command
 $rlsCmdout        Stores o/p from command if successful
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub ExecLocalCmd {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sUser,
       $sEnvFile,
       $sCommand,
       $sCommandArgs,
       $riExitCode,
       $iTimeout,
       $rlsCmdout ,
       $sOutputFilename ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20117, "Subroutine <%s> called with:
  User        [%s]
  EnvFile     [%s]
  Command     [%s]
  CommandArgs [%s]
  riExitCode  [%s]
  Timeout     [%s]
  rlsCmdout   [%s]",

  $sMyName,
  $sUser, $sEnvFile, $sCommand, $sCommandArgs, $riExitCode,
  $iTimeout, $rlsCmdout );

  #---------------------------------------
  # Setup return code to hopeful success..
  #---------------------------------------
  my $iReturnCode = 1;

  

  #-------------------
  # Set up arguments..
  #-------------------
  my $bSourceUserEnv = 0; # Do not source..
  my $bTimedOut = 0;      # Timeout variable..

  @$rlsCmdout = (); # Reset..

  #-----------
  # Validate..
  #-----------
  if ( ! -e $sCommand ) {
    &logError ( 20118,
    "<%s> Cmd [%s] does not exist", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( ! -x $sCommand ) {
    &logError ( 20119,
    "<%s> Cmd [%s] is not executable", $sMyName, $sCommand );
    $iReturnCode = 0; # Failure..
  }
  elsif ( $iTimeout <= 1 ) {
    &logError ( 20120,
    "<%s> Insufficient time [%s] to execute Cmd [%s] with Args [%s].",
    $sMyName, $iTimeout, $sCommand, $sCommandArgs );
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return if failed.. 

  #----------------------
  # Set LOGNAME to user..
  #----------------------
  my $sOldLogName = $ENV { LOGNAME }; # Backup..
  $ENV { LOGNAME } = $sUser;          # Now set..

  &logTrace ( 20121,
  "<%s> Executing Command [%s] with Args [%s]",
  $sMyName, $sCommand, $sCommandArgs );

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut ( $sEnvFile,
                                                           $bSourceUserEnv,
                                                           $sCommand,
                                                           $sCommandArgs,
                                                           $riExitCode,
                                                           $sUser,
                                                           $rlsCmdout,
                                                           \$bTimedOut,
                                                           $iTimeout,
                                                           $sOutputFilename,
                                                           dirname($sCommand) );

  #---------------------------
  # Remove trailing newlines..
  #---------------------------
  chomp @$rlsCmdout;

  #--------------------
  # Check return code..
  #--------------------
  if ( $iReturnCode ) {   # No error with command execution..
    if ( $$riExitCode ) { # Failure..
      &logTrace ( 20122,
      "<%s> Command [%s] with args [%s] returned exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
      $iReturnCode = 0; # Failure..
    }
    else {
      &logTrace ( 20123,
      "<%s> Command [%s] with args [%s] executed successfully with exit code [%s]",
      $sMyName, $sCommand, $sCommandArgs, $$riExitCode );
    }
  }
  else { # Execution error..
    @$rlsCmdout = (); # Reset..

    if ( $bTimedOut ) {  # We timed out..
      &logError ( 20124,
      "<%s> Command [%s] with args [%s] did not complete within [%s] seconds",
      $sMyName, $sCommand, $sCommandArgs, $iTimeout );
      $$rlsCmdout [0] = 'timeout'; # Timeout..
    }
    else { # Failure due to something else..
      &logError ( 20125,
      "<%s> Failed in call to RunWithEnvCmdWithOutputWithTimeOut",
      $sMyName );
    }
  }

  #----------
  # Restore..
  #----------
  defined $sOldLogName ? $ENV { LOGNAME } = $sOldLogName : delete $ENV { LOGNAME };

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : bool GetPidDetails ( IN Integer Pid,
                                 OUT RefList ProcList );


 Description : Gets long process details for a known Pid if running

 Parameters :

 $iPid         PID whose details are requested
 $rlProcList   Ref to List to store details
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub GetPidDetails {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iPid,
       $rlProcList ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20126, "Subroutine <%s> called with:
  Pid             [%s]
  rlProcList      [%s]",

  $sMyName, $iPid, $rlProcList );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #---------------
  # Validate PID..
  #---------------
  unless ( defined $iPid    &&   # Catch undefs..
           $iPid =~ /^\d+$/ &&   # And invalid PIDs..
           kill ( 0, $iPid ) ) { # And defunct ones..
    &logError ( 20127,
    "<%s> PID [%s] is invalid", $sMyName, $iPid );
    return 0;
  }

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sPsCmd, $sPsCmdArgs, $sLang, $sFmt, $sUnix95, $sSwitch, $iExitCode,
       %hsProc ) = ();

  $sSwitch = $^O;    # Get OS..
  @$rlProcList = (); # Truncate..

  #-------------
  # Store LANG..
  #-------------
  $sLang = $ENV { LANG };
  $ENV { LANG } = 'C';

  #---------------------------
  # Switch to the correct OS..
  #---------------------------
  if ( "solaris" eq $sSwitch ) {
      my $sOSVer = ( uname ) [2]; # Get OS version..
      if ($sOSVer =~ /\d+\.(\d+)/) {
           $sOSVer = $1;
      }
      $sPsCmd = "/usr/bin/ps";
      if ( $sOSVer < 11 ) {
         $sPsCmd = "/usr/ucb/ps" if ( -x "/usr/ucb/ps" );
      }
      $sPsCmdArgs = "axwwl $iPid";
      $sFmt = '[\da-z]+\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)';
  }
  elsif ( "hpux" eq $sSwitch ) {
      #-----------------------
      # Force XPG4 behaviour..
      #-----------------------
      $sUnix95 = $ENV { UNIX95 };
      $ENV { UNIX95 } = 'XPG4';

      $sPsCmd = "/usr/bin/ps";
      $sPsCmdArgs = "-p $iPid -xxlo uid,pid,ppid,args";
      $sFmt = '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)';
  }
  elsif ( "aix" eq $sSwitch ) {
      $sPsCmd = "/usr/bin/ps";
      $sPsCmdArgs = "axwwl $iPid";
      $sFmt =  '[\da-e]+\s+[A-Z]\s+([\d]+)\s+([\d]+)\s+([\d]+)\s+.*?[\d]:[\d]{2}\s(.*)';
  }
  elsif ( "linux" eq $sSwitch ) {
      $sPsCmd = "/bin/ps";
      $sPsCmdArgs = "wwo uid,pid,ppid,args $iPid";
      $sFmt = '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)';
  }

  &logTrace ( 20128, "<%s> OS [%s] PsCmd [%s] PsCmdArgs [%s]",
  $sMyName, $sSwitch, $sPsCmd, $sPsCmdArgs );

  #-------------------
  # Validate command..
  #-------------------
  if ( ! -x $sPsCmd ) {
    &logError ( 20129,
    "<%s> ps(1b) command [%s] is not executable", $sMyName, $sPsCmd );
    return 0; # Failure..
  }

  #-------------------------------
  # Fire command and get details..
  #-------------------------------
  @hsProc { qw ( UID PID PPID COMMAND ) } =
  map {
    s/\n//;                             # Chomp..
    s/^\s+//;                           # Strip leading spaces..
    /^$sFmt$/o;                         # Extract fields..
  } readpipe ( "$sPsCmd $sPsCmdArgs" ); # From the command..

  $iExitCode = $? >> 8; # Get exit code..

  #------------------------
  # Restore env variables..
  #------------------------
  defined $sLang ? $ENV { LANG } = $sLang : delete $ENV { LANG };
  defined $sUnix95 ? $ENV { UNIX95 } = $sUnix95 : delete $ENV { UNIX95 };

  #------------------
  # Check exit code..
  #------------------
  if ( $iExitCode ) {
    &logError ( 20130,
    "<%s> Cmd [%s] with args [%s] failed with exit code [%s]",
    $sMyName, $sPsCmd, $sPsCmdArgs, $iExitCode );

    %hsProc = (); # Truncate..
    return 0;
  }

  #---------------------
  # Fill up the bucket..
  #---------------------
  push ( @$rlProcList, \%hsProc ); # Update..

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool GetSrvrDetails ( IN String ActiveMQHome,
                                  IN String DXHome,
                                  IN String Hostname,
                                  IN String InfaHome,
                                  IN String MFTHome,
                                  OUT RefHashList SrvrDetails );

 Description : Gives PID file and process string for given server type

 Parameters  :

 $sActiveMQHome             Absolute path to the ActiveMQ component's installation directory
 $sDXHome                   Absolute path to the B2B Data Exchange installation directory
 $sHostname                 Hostname on which the resource is configured
 $sInfaHome                 Absolute path to the PowerCenter installation directory.
 $sMFTHome                  Absolute path to the Managed File Transfer component's installation directory.
 $rhlSrvrDetails            Stores details of PID file and process string
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub GetSrvrDetails {
  #----------------
  # Get Arguments..
  #----------------
  my ( $sActiveMQHome,
       $sDXHome,
       $sHostname,
       $sInfaHome,
       $sMFTHome,
       $sComponent,
       $rhlSrvrDetails ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20131, "Subroutine <%s> called with:
  ActiveMQHome             [%s]
  DXHome                   [%s]
  Hostname                 [%s]
  InfaHome                 [%s]
  MFTHome                  [%s]
  Component                [%s]
  SrvrDetails              [%s]",

  $sMyName,  $sActiveMQHome,  $sDXHome,   $sHostname,   $sInfaHome,
  $sMFTHome, $sComponent, $rhlSrvrDetails );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = 1; # Set to true..

  #-------------------
  # Define variables..
  #-------------------
  my ( $sKey, $sProc, $sPidFile, $sPIDDir, $sResourceType,
       $sResourceName, $iPat ) = ();

  #-------------------------------------
  # List of search patterns to monitor..
  #-------------------------------------
  #activemq: "java .*-jar $ActiveMQHome/bin/run.jar start"
  ##dxconsole: "java .*-classpath.*$sDXHome\b.*-Dcatalina.home=$sDXHome/apache-tomcat-\S+\b.*org.apache.catalina.startup.Bootstrap start"
  ##dxserver: "java .*-classpath .:$sDXHome:.*com.informatica.b2b.dx.broker.DXControl start"
  ##mft: java .* com.zerog.lax.LAX $MFTHome/VLTraderc.lax
  #
  my @lsSrchPatterns = ();
  if ($sComponent eq "activemq") {
    @lsSrchPatterns = ( qr!java .*-jar $sActiveMQHome/bin/(run|activemq).jar start! );
  } elsif ($sComponent eq "dxconsole") {
    @lsSrchPatterns = ( qr!java .*-classpath.*$sDXHome\b.*-Dcatalina.home=$sDXHome/(apache-tomcat-\S+\b|tomcat\b).*Bootstrap start! );
  } elsif ($sComponent eq "dxserver") {
    @lsSrchPatterns = ( qr!java .*-classpath .:$sDXHome:.*com.informatica.b2b.dx.broker.DXControl start! );
  } elsif ($sComponent eq "mft") {
    @lsSrchPatterns = ( qr!java .* com.zerog.lax.LAX $sMFTHome/VLTraderc.lax! );
  } elsif ($sComponent eq "powercenter") {
    @lsSrchPatterns = ( qr!$sInfaHome/java/bin/java.*java\.awt\.headless=true\b|bin/java .*$sInfaHome\b.*java\.awt\.headless=true\b|bin/java .*java\.awt\.headless=true.*$sInfaHome\b! );

    my $sEntryPointName = "";
    ( $sEntryPointName = $0 ) =~ s#.*/##; # Basename..

    if ( "clean" eq $sEntryPointName || "offline" eq $sEntryPointName ) {
	 push ( @lsSrchPatterns, qr!java .*-DINFA_HOME=$sInfaHome\b.*com.informatica.isp.tools.tomcatstarter.InfaTomcatBootstrap start!);
	 push ( @lsSrchPatterns, qr!java .*-Dprogram.name=run.sh\b.*-DINFA_HOME=$sInfaHome\b.*-classpath\s+$sInfaHome/\S+/jboss/bin/run.jar.*org.jboss.Main -c! );

    }
  }

  #----------------------------------------------
  # Derive our resource type and resource names..
  #----------------------------------------------
  $sResourceType = ( $0 =~ m@([^/]+)@g ) [-2];
  $sResourceName   = $ENV { VCS_LOG_RESOURCE_NAME };
  $sPIDDir = "/tmp/.VRTS$sResourceType/$sResourceName/PID";

  my $sOldMask = umask ( 000 );

  #------------------------------
  # Create PID folder if needed..
  #------------------------------
  if ( -d $sPIDDir ) {
    &logTrace ( 20132, "<%s> PID folder [%s] exists",
    $sMyName, $sPIDDir );
    chmod (0777, "/tmp/.VRTS$sResourceType");
  }
  elsif ( ! mkpath ( $sPIDDir, 0, 0777 ) ) {
    &logWarn ( 20133, "<%s> Failed to create PID folder [%s]",
    $sMyName, $sPIDDir );
    return 0; # Cannot proceed..
  }

  umask ( $sOldMask );
  %$rhlSrvrDetails = (); # Truncate..

  #-----------------------------------------------
  # Set appropriate filter strings and PID files..
  #-----------------------------------------------
  foreach $sProc ( @lsSrchPatterns ) {
    $sKey = 'srchstr_' . ++ $iPat;
    $sPidFile = "$sPIDDir/pidfile.$sKey";
    @{ $$rhlSrvrDetails { $sKey } } = ( $sPidFile, $sProc ); # Store..
  }

  return $iReturnCode;
}

sub HTTPConnect
{
  #----------------
  # Get Arguments..
  #----------------
  my ( $sHost,
       $iPort,
       $sUrl ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20134, "Subroutine <%s> called with:
  Host                     [%s]
  Port                     [%s]
  Url                      [%s]",
  $sMyName,  $sHost, $iPort, $sUrl );

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = 1; # Set to true..

  my ($sFullUrl, $reqHTTP, $uaHTTP, $respHTTP, $iResponseCode ) = ();

  $sFullUrl = 'http://' . $sHost . ':' . $iPort . $sUrl;

  &logTrace ( 20135,
  "<%s> Querying HTTP Url: [%s]",
  $sMyName, $sFullUrl );

  $reqHTTP = HTTP::Request->new(GET => $sFullUrl);

  $uaHTTP = LWP::UserAgent->new;
  $uaHTTP->timeout(5);
  $uaHTTP->agent("");

  $respHTTP = $uaHTTP->request($reqHTTP);

  $iResponseCode = $respHTTP->code ();
  return $iResponseCode;
}

1; # So PERL knows its included..
